﻿using IceCreamBusinessLogic.OfficePackage.HelperEnums;
using IceCreamBusinessLogic.OfficePackage.HelperModels;

namespace IceCreamBusinessLogic.OfficePackage
{
    public abstract class AbstractSaveToWord
    {
        public void CreateDoc(WordInfo info)
        {
            CreateWord(info);

            CreateParagraph(new WordParagraph
            {
                Texts = new List<(string, WordTextProperties)> { (info.Title, new WordTextProperties { Bold = true, Size = "24", }) },
                TextProperties = new WordTextProperties
                {
                    Size = "24",
                    JustificationType = WordJustificationType.Center
                }
            });

            foreach (var iceCream in info.IceCreams)
            {
                CreateParagraph(new WordParagraph
                {
                    Texts = new List<(string, WordTextProperties)> {
                        (iceCream.IceCreamName, new WordTextProperties { Size = "24", Bold = true}),
                        (" - цена " + iceCream.Price.ToString(), new WordTextProperties { Size = "24" })
                    },
                    TextProperties = new WordTextProperties
                    {
                        Size = "24",
                        JustificationType = WordJustificationType.Both
                    }
                });
            }

            SaveWord(info);
        }

        public void CreateTableDoc(WordInfo wordInfo)
        {
            CreateWord(wordInfo);
            var list = new List<string>();
            foreach (var shop in wordInfo.Shops)
            {
                list.Add(shop.Name);
                list.Add(shop.Adress);
                list.Add(shop.OpeningDate.ToString());
            }
            var wordTable = new WordTable
            {
                Headers = new List<string> {
                    "Название",
                    "Адрес",
                    "Дата открытия"},
                Texts = list
            };
            CreateTable(wordTable);

            SaveWord(wordInfo);

        }

        /// <summary>
		/// Создание doc-файла
		/// </summary>
		/// <param name="info"></param>
        protected abstract void CreateWord(WordInfo info);

        /// <summary>
        /// Создание абзаца с текстом
        /// </summary>
        /// <param name="paragraph"></param>
        /// <returns></returns>
        protected abstract void CreateParagraph(WordParagraph paragraph);

        /// <summary>
		/// Сохранение файла
		/// </summary>
		/// <param name="info"></param>
        protected abstract void SaveWord(WordInfo info);
        protected abstract void CreateTable(WordTable info);
    }
}
