﻿using AbstractIceCreamShopDataModels.Enums;
using AbstractIceCreamShopDataModels.Models;
using IceCreamShopContracts.BindingModels;
using IceCreamShopContracts.ViewModels;
using System.ComponentModel.DataAnnotations;
using System.Reflection.Metadata;

namespace IceCreamShopDatabaseImplement.Models
{
    public class Order : IOrderModel
    {
        [Required]
        public int IceCreamId { get; set; }
        [Required]
        public int ClientId { get; private set; }
        public int? ImplementerId { get; private set; }
        [Required]
        public int Count { get; set; }
        [Required]
        public double Sum { get; set; }
        [Required]
        public OrderStatus Status { get; set; }
        [Required]
        public DateTime DateCreate { get; set; }

        public DateTime? DateImplement { get; set; }

        public int Id { get; set; }
        public virtual IceCream IceCream{ get; private set; }
        public virtual Client Client { get; private set; }
        public Implementer? Implementer { get; private set; }
        public static Order? Create(OrderBindingModel? model)
        {
            if (model == null)
            {
                return null;
            }
            return new Order
            {
                IceCreamId = model.IceCreamId,
                ClientId = model.ClientId,
                ImplementerId = model.ImplementerId,
                Count = model.Count,
                Sum = model.Sum,
                Status = model.Status,
                DateCreate = model.DateCreate,
                DateImplement = model.DateImplement,
                Id = model.Id,
            };
        }

        public void Update(OrderBindingModel? model)
        {
            if (model == null)
            {
                return;
            }
            Status = model.Status;
            ImplementerId = model.ImplementerId;
            DateImplement = model.DateImplement;
        }

        public OrderViewModel GetViewModel
        {
            get
            {
                var context = new IceCreamShopDatabase();
                return new()
                {
                    IceCreamId = IceCreamId,
                    ClientId = ClientId,
                    Count = Count,
                    ImplementerId = ImplementerId,
                    Sum = Sum,
                    Status = Status,
                    DateCreate = DateCreate,
                    DateImplement = DateImplement,
                    Id = Id,
                    IceCreamName = context.IceCreams.FirstOrDefault(x => x.Id == IceCreamId)?.IceCreamName ?? string.Empty,
                    ImplementerFIO = Implementer?.ImplementerFIO ?? string.Empty,
                    ClientFIO = Client?.ClientFIO ?? string.Empty,
                };
            }
        }
    }
}
